@@ -33,7 +33,7 @@ from six import iteritems
 
 from toil.lib.humanize import bytes2human
 from toil.lib.retry import retry
-from toil import subprocess
+import subprocess
 from toil import pickle
 from toil import logProcessContext
 from toil.lib.bioio import addLoggingOptions, getLogLevelString, setLoggingFromOptions
@@ -114,6 +114,7 @@ class Config(object):
 
         # Retrying/rescuing jobs
         self.retryCount = 1
+        self.enableUnlimitedPreemptableRetries = False
         self.maxJobDuration = sys.maxsize
         self.rescueJobsFrequency = 3600
 
@@ -220,6 +221,7 @@ class Config(object):
         setOption("parasolCommand")
         setOption("parasolMaxBatches", int, iC(1))
         setOption("linkImports")
+        setOption("moveExports")
         setOption("environment", parseSetEnv)
 
         # Autoscaling options
@@ -262,6 +264,7 @@ class Config(object):
 
         # Retrying/rescuing jobs
         setOption("retryCount", int, iC(1))
+        setOption("enableUnlimitedPreemptableRetries")
         setOption("maxJobDuration", int, iC(1))
         setOption("rescueJobsFrequency", int, iC(1))
 
@@ -327,13 +330,16 @@ def _addOptions(addGroupFn, config):
                 help="The location of the job store for the workflow. " + jobStoreLocatorHelp)
     addOptionFn("--workDir", dest="workDir", default=None,
                 help="Absolute path to directory where temporary files generated during the Toil "
-                     "run should be placed. Temp files and folders, as well as standard output "
-                     "and error from batch system jobs (unless --noStdOutErr), will be placed in a "
+                     "run should be placed. Standard output and error from batch system jobs "
+                     "(unless --noStdOutErr) will be placed in this directory. A cache directory "
+                     "may be placed in this directory. Temp files and folders will be placed in a "
                      "directory toil-<workflowID> within workDir. The workflowID is generated by "
                      "Toil and will be reported in the workflow logs. Default is determined by the "
                      "variables (TMPDIR, TEMP, TMP) via mkdtemp. This directory needs to exist on "
                      "all machines running jobs; if capturing standard output and error from batch "
-                     "system jobs is desired, it will generally need to be on a shared file system.")
+                     "system jobs is desired, it will generally need to be on a shared file system. "
+                     "When sharing a cache between containers on a host, this directory must be "
+                     "shared between the containers.")
     addOptionFn("--noStdOutErr", dest="noStdOutErr", action="store_true", default=None,
                 help="Do not capture standard output and error from batch system jobs.")
     addOptionFn("--stats", dest="stats", action="store_true", default=None,
@@ -520,6 +526,11 @@ def _addOptions(addGroupFn, config):
     addOptionFn("--retryCount", dest="retryCount", default=None,
                 help=("Number of times to retry a failing job before giving up and "
                       "labeling job failed. default=%s" % config.retryCount))
+    addOptionFn("--enableUnlimitedPreemptableRetries", dest="enableUnlimitedPreemptableRetries",
+                action='store_true', default=False,
+                help=("If set, preemptable failures (or any failure due to an instance getting "
+                      "unexpectedly terminated) would not count towards job failures and "
+                      "--retryCount."))
     addOptionFn("--maxJobDuration", dest="maxJobDuration", default=None,
                 help=("Maximum runtime of a job (in seconds) before we kill it "
                       "(this is a lower bound, and the actual time before killing "
@@ -919,7 +930,7 @@ class Toil(object):
                                '--disableCaching flag if you want to '
                                'use this batch system.' % config.batchSystem)
         logger.debug('Using the %s' %
-                    re.sub("([a-z])([A-Z])", "\g<1> \g<2>", batchSystemClass.__name__).lower())
+                    re.sub("([a-z])([A-Z])", r'\g<1> \g<2>', batchSystemClass.__name__).lower())
 
         return batchSystemClass(**kwargs)
 
@@ -947,7 +958,7 @@ class Toil(object):
                 else:
                     from toil.batchSystems.singleMachine import SingleMachineBatchSystem
                     if not isinstance(self._batchSystem, SingleMachineBatchSystem):
-                        logger.warn('Batch system does not support auto-deployment. The user '
+                        logger.warning('Batch system does not support auto-deployment. The user '
                                     'script %s will have to be present at the same location on '
                                     'every worker.', userScript)
                     userScript = None
@@ -1022,23 +1033,44 @@ class Toil(object):
         self._jobCache[job.jobStoreID] = job
 
     @staticmethod
-    def getWorkflowDir(workflowID, configWorkDir=None):
+    def getToilWorkDir(configWorkDir=None):
         """
-        Returns a path to the directory where worker directories and the cache will be located
-        for this workflow.
+        Returns a path to a writable directory under which per-workflow
+        directories exist.  This directory is always required to exist on a
+        machine, even if the Toil worker has not run yet.  If your workers and
+        leader have different temp directories, you may need to set
+        TOIL_WORKDIR.
 
-        :param str workflowID: Unique identifier for the workflow
         :param str configWorkDir: Value passed to the program using the --workDir flag
-        :return: Path to the workflow directory
+        :return: Path to the Toil work directory, constant across all machines
         :rtype: str
         """
+
         workDir = configWorkDir or os.getenv('TOIL_WORKDIR') or tempfile.gettempdir()
         if not os.path.exists(workDir):
             raise RuntimeError("The directory specified by --workDir or TOIL_WORKDIR (%s) does not "
                                "exist." % workDir)
-        # Create the workflow dir, make it unique to each host in case workDir is on a shared FS.
-        # This prevents workers on different nodes from erasing each other's directories.
-        workflowDir = os.path.join(workDir, 'toil-%s-%s' % (workflowID, getNodeID()))
+        return workDir
+
+    @classmethod
+    def getLocalWorkflowDir(cls, workflowID, configWorkDir=None):
+        """
+        Returns a path to the directory where worker directories and the cache will be located
+        for this workflow on this machine.
+
+        :param str workflowID: Unique identifier for the workflow
+        :param str configWorkDir: Value passed to the program using the --workDir flag
+        :return: Path to the local workflow directory on this machine
+        :rtype: str
+        """
+
+        # Get the global Toil work directory. This ensures that it exists.
+        base = cls.getToilWorkDir(configWorkDir=configWorkDir)
+
+        # Create a directory unique to each host and workflow in case workDir
+        # is on a shared FS. This prevents workers on different nodes from
+        # erasing each other's directories.
+        workflowDir = os.path.join(base, 'node-%s-%s' % (workflowID, getNodeID()))
         try:
             # Directory creation is atomic
             os.mkdir(workflowDir)
@@ -1047,7 +1079,7 @@ class Toil(object):
                 # The directory exists if a previous worker set it up.
                 raise
         else:
-            logger.debug('Created the workflow directory at %s' % workflowDir)
+            logger.debug('Created the workflow directory for this machine at %s' % workflowDir)
         return workflowDir
 
     def _runMainLoop(self, rootJob):
@@ -1146,7 +1178,7 @@ class ToilMetrics:
                                                self.mtailImage],
                                               stdin=subprocess.PIPE, stdout=subprocess.PIPE)
         except subprocess.CalledProcessError:
-            logger.warn("Could not start toil metrics server.")
+            logger.warning("Could not start toil metrics server.")
             self.mtailProc = None
         except KeyboardInterrupt:
             self.mtailProc.terminate()
@@ -1168,7 +1200,7 @@ class ToilMetrics:
                                                           "-collector.filesystem.ignored-mount-points",
                                                           "^/(sys|proc|dev|host|etc)($|/)"])
             except subprocess.CalledProcessError:
-                logger.warn(
+                logger.warning(
                     "Couldn't start node exporter, won't get RAM and CPU usage for dashboard.")
                 self.nodeExporterProc = None
             except KeyboardInterrupt:
@@ -1209,7 +1241,7 @@ class ToilMetrics:
                                        "-d", "-p=3000:3000",
                                        self.grafanaImage])
         except subprocess.CalledProcessError:
-            logger.warn("Could not start prometheus/grafana dashboard.")
+            logger.warning("Could not start prometheus/grafana dashboard.")
             return
 
         # Add prometheus data source
@@ -1231,7 +1263,8 @@ class ToilMetrics:
 
     def log(self, message):
         if self.mtailProc:
-            self.mtailProc.stdin.write(message + "\n")
+            self.mtailProc.stdin.write((message + "\n").encode("utf-8"))
+            self.mtailProc.stdin.flush()
 
     # Note: The mtail configuration (dashboard/mtail/toil.mtail) depends on these messages
     # remaining intact
