@@ -69,22 +69,28 @@ logger = logging.getLogger( __name__ )
 
 class FailedJobsException(Exception):
     def __init__(self, jobStoreLocator, failedJobs, jobStore):
-        msg = "The job store '%s' contains %i failed jobs" % (jobStoreLocator, len(failedJobs))
+        self.msg = "The job store '%s' contains %i failed jobs" % (jobStoreLocator, len(failedJobs))
         try:
-            msg += ": %s" % ", ".join((str(failedJob) for failedJob in failedJobs))
+            self.msg += ": %s" % ", ".join((str(failedJob) for failedJob in failedJobs))
             for jobNode in failedJobs:
                 job = jobStore.load(jobNode.jobStoreID)
                 if job.logJobStoreFileID:
-                    msg += "\n=========> Failed job %s \n" % jobNode
                     with job.getLogFileHandle(jobStore) as fH:
-                        msg += fH.read().decode('utf-8')
-                    msg += "<=========\n"
+                        self.msg += "\n" + StatsAndLogging.formatLogStream(fH, jobNode)
         # catch failures to prepare more complex details and only return the basics
         except:
             logger.exception('Exception when compiling information about failed jobs')
+        self.msg = self.msg.rstrip('\n')
         super().__init__()
         self.jobStoreLocator = jobStoreLocator
         self.numberOfFailedJobs = len(failedJobs)
+    
+    def __str__(self):
+        """
+        Stringify the exception, including the message.
+        """
+        return self.msg
+    
 
 ####################################################
 # Exception thrown by the Leader class when a deadlock is encountered due to insufficient
@@ -93,9 +99,15 @@ class FailedJobsException(Exception):
 
 class DeadlockException(Exception):
     def __init__(self, msg):
-        msg = "Deadlock encountered: " + msg
+        self.msg = "Deadlock encountered: " + msg
         super().__init__()
 
+    def __str__(self):
+        """
+        Stringify the exception, including the message.
+        """
+        return self.msg
+
 ####################################################
 ##Following class represents the leader
 ####################################################
@@ -186,8 +198,7 @@ class Leader(object):
         """
         This runs the leader process to issue and manage jobs.
 
-        :raises: toil.leader.FailedJobsException if at the end of function their remain \
-        failed jobs
+        :raises: toil.leader.FailedJobsException if failed jobs remain after running.
 
         :return: The return value of the root job's run function.
         :rtype: Any
@@ -369,7 +380,7 @@ class Leader(object):
             # been tried once (without decreasing its retry count as the job
             # itself was successful), and its subtree failed, it shouldn't be retried
             # unless it has more than 1 try.
-            logger.warn('Job: %s is being restarted as a checkpoint after the total '
+            logger.warning('Job: %s is being restarted as a checkpoint after the total '
                         'failure of jobs in its subtree.', jobGraph.jobStoreID)
             self.issueJob(JobNode.fromJobGraph(jobGraph))
         else:
@@ -401,7 +412,7 @@ class Leader(object):
             if (jobGraph.remainingRetryCount == 0
                 or isServiceJob and not self.jobStore.fileExists(jobGraph.errorJobStoreID)):
                 self.processTotallyFailedJob(jobGraph)
-                logger.warn("Job %s with ID %s is completely failed",
+                logger.warning("Job %s with ID %s is completely failed",
                             jobGraph, jobGraph.jobStoreID)
             else:
                 # Otherwise try the job again
@@ -444,7 +455,7 @@ class Leader(object):
                 logger.debug("Job: %s is empty, we are scheduling to clean it up", jobGraph.jobStoreID)
             else:
                 self.processTotallyFailedJob(jobGraph)
-                logger.warn("Job: %s is empty but completely failed - something is very wrong", jobGraph.jobStoreID)
+                logger.warning("Job: %s is empty but completely failed - something is very wrong", jobGraph.jobStoreID)
 
     def _processReadyJobs(self):
         """Process jobs that are ready to be scheduled/have successors to schedule"""
@@ -480,24 +491,26 @@ class Leader(object):
 
     def _gatherUpdatedJobs(self, updatedJobTuple):
         """Gather any new, updated jobGraph from the batch system"""
-        jobID, result, wallTime = updatedJobTuple
+        jobID, exitStatus, exitReason, wallTime = (
+            updatedJobTuple.jobID, updatedJobTuple.exitStatus, updatedJobTuple.exitReason,
+            updatedJobTuple.wallTime)
         # easy, track different state
         try:
             updatedJob = self.jobBatchSystemIDToIssuedJob[jobID]
         except KeyError:
-            logger.warn("A result seems to already have been processed "
+            logger.warning("A result seems to already have been processed "
                         "for job %s", jobID)
         else:
-            if result == 0:
+            if exitStatus == 0:
                 cur_logger = (logger.debug if str(updatedJob.jobName).startswith(CWL_INTERNAL_JOBS)
                               else logger.info)
                 cur_logger('Job ended: %s', updatedJob)
                 if self.toilMetrics:
                     self.toilMetrics.logCompletedJob(updatedJob)
             else:
-                logger.warn('Job failed with exit value %i: %s',
-                            result, updatedJob)
-            self.processFinishedJob(jobID, result, wallTime=wallTime)
+                logger.warning('Job failed with exit value %i: %s',
+                               exitStatus, updatedJob)
+            self.processFinishedJob(jobID, exitStatus, wallTime=wallTime, exitReason=exitReason)
 
     def _processLostJobs(self):
         """Process jobs that have gone awry"""
@@ -710,7 +723,7 @@ class Leader(object):
             runningJobs = self.batchSystem.getRunningBatchJobIDs()
             for jobBatchSystemID in list(runningJobs.keys()):
                 if runningJobs[jobBatchSystemID] > maxJobDuration:
-                    logger.warn("The job: %s has been running for: %s seconds, more than the "
+                    logger.warning("The job: %s has been running for: %s seconds, more than the "
                                 "max job duration: %s, we'll kill it",
                                 str(self.jobBatchSystemIDToIssuedJob[jobBatchSystemID].jobStoreID),
                                 str(runningJobs[jobBatchSystemID]),
@@ -731,7 +744,7 @@ class Leader(object):
         missingJobIDsSet = set(list(self.reissueMissingJobs_missingHash.keys()))
         for jobBatchSystemID in missingJobIDsSet.difference(jobBatchSystemIDsSet):
             self.reissueMissingJobs_missingHash.pop(jobBatchSystemID)
-            logger.warn("Batch system id: %s is no longer missing", str(jobBatchSystemID))
+            logger.warning("Batch system id: %s is no longer missing", str(jobBatchSystemID))
         assert runningJobs.issubset(jobBatchSystemIDsSet) #Assert checks we have
         #no unexpected jobs running
         jobsToKill = []
@@ -742,7 +755,7 @@ class Leader(object):
             else:
                 self.reissueMissingJobs_missingHash[jobBatchSystemID] = 1
             timesMissing = self.reissueMissingJobs_missingHash[jobBatchSystemID]
-            logger.warn("Job store ID %s with batch system id %s is missing for the %i time",
+            logger.warning("Job store ID %s with batch system id %s is missing for the %i time",
                         jobStoreID, str(jobBatchSystemID), timesMissing)
             if self.toilMetrics:
                 self.toilMetrics.logMissingJob()
@@ -755,11 +768,11 @@ class Leader(object):
 
     def processRemovedJob(self, issuedJob, resultStatus):
         if resultStatus != 0:
-            logger.warn("Despite the batch system claiming failure the "
+            logger.warning("Despite the batch system claiming failure the "
                         "job %s seems to have finished and been removed", issuedJob)
         self._updatePredecessorStatus(issuedJob.jobStoreID)
 
-    def processFinishedJob(self, batchSystemID, resultStatus, wallTime=None):
+    def processFinishedJob(self, batchSystemID, resultStatus, wallTime=None, exitReason=None):
         """
         Function reads a processed jobGraph file and updates its state.
         """
@@ -779,7 +792,7 @@ class Leader(object):
                     # Process the job from here as any other job removed from the job store.
                     # This is a temporary work around until https://github.com/BD2KGenomics/toil/issues/1091
                     # is completed
-                    logger.warn('Got a stale read from SDB for job %s', jobNode)
+                    logger.warning('Got a stale read from SDB for job %s', jobNode)
                     self.processRemovedJob(jobNode, resultStatus)
                     return
                 else:
@@ -789,7 +802,7 @@ class Leader(object):
                     # more memory efficient than read().striplines() while leaving off the
                     # trailing \n left when using readlines()
                     # http://stackoverflow.com/a/15233739
-                    StatsAndLogging.logWithFormatting(jobStoreID, logFileStream, method=logger.warn,
+                    StatsAndLogging.logWithFormatting(jobStoreID, logFileStream, method=logger.warning,
                                                       message='The job seems to have left a log file, indicating failure: %s' % jobGraph)
                 if self.config.writeLogs or self.config.writeLogsGzip:
                     with jobGraph.getLogFileHandle(self.jobStore) as logFileStream:
@@ -799,22 +812,28 @@ class Leader(object):
                 # is assumed not to have captured the failure of the job, so we
                 # reduce the retry count here.
                 if jobGraph.logJobStoreFileID is None:
-                    logger.warn("No log file is present, despite job failing: %s", jobNode)
+                    logger.warning("No log file is present, despite job failing: %s", jobNode)
 
-                # Look for any standard output/error files created by the batch system
-                workflowDir = Toil.getWorkflowDir(self.config.workflowID, self.config.workDir)
-                batchSystemFilePrefix = 'toil_job_{}_batch_'.format(batchSystemID)
-                batchSystemFileGlob = os.path.join(workflowDir, batchSystemFilePrefix + '*.log')
+                # Look for any standard output/error files created by the batch system.
+                # They will only appear if the batch system actually supports
+                # returning logs to the machine that submitted jobs, or if
+                # --workDir / TOIL_WORKDIR is on a shared file system.
+                # They live directly in the Toil work directory because that is
+                # guaranteed to exist on the leader and workers.
+                workDir = Toil.getToilWorkDir(self.config.workDir)
+                # This must match the format in AbstractBatchSystem.formatStdOutErrPath()
+                batchSystemFilePrefix = 'toil_workflow_{}_job_{}_batch_'.format(self.config.workflowID, batchSystemID)
+                batchSystemFileGlob = os.path.join(workDir, batchSystemFilePrefix + '*.log')
                 batchSystemFiles = glob.glob(batchSystemFileGlob)
                 for batchSystemFile in batchSystemFiles:
                     try:
                         batchSystemFileStream = open(batchSystemFile, 'rb')
                     except:
-                        logger.warn('The batch system left a file %s, but it could not be opened' % batchSystemFile)
+                        logger.warning('The batch system left a file %s, but it could not be opened' % batchSystemFile)
                     else:
                         with batchSystemFileStream:
                             if os.path.getsize(batchSystemFile) > 0:
-                                StatsAndLogging.logWithFormatting(jobStoreID, batchSystemFileStream, method=logger.warn,
+                                StatsAndLogging.logWithFormatting(jobStoreID, batchSystemFileStream, method=logger.warning,
                                                                   message='The batch system left a non-empty file %s:' % batchSystemFile)
                                 if self.config.writeLogs or self.config.writeLogsGzip:
                                     batchSystemFileRoot, _ = os.path.splitext(os.path.basename(batchSystemFile))
@@ -825,9 +844,9 @@ class Leader(object):
                                     batchSystemFileStream.seek(0)
                                     StatsAndLogging.writeLogFiles(jobNames, batchSystemFileStream, self.config, failed=True)
                             else:
-                                logger.warn('The batch system left an empty file %s' % batchSystemFile)
+                                logger.warning('The batch system left an empty file %s' % batchSystemFile)
 
-                jobGraph.setupJobAfterFailure(self.config)
+                jobGraph.setupJobAfterFailure(self.config, exitReason=exitReason)
                 self.jobStore.update(jobGraph)
             elif jobStoreID in self.toilState.hasFailedSuccessors:
                 # If the job has completed okay, we can remove it from the list of jobs with failed successors
